<?php

/**
 * @file
 * Hit details report for the visitors module.
 */

/**
 * Menu callback; Displays recent page accesses.
 *
 * @param visitors_id
 *   int visitors id from visitors table
 *
 * @return
 *   string hit details report or 404 error if visitors_id not found
 */
function visitors_hit_details($visitors_id) {
  $hit_details = visitors_hit_details_get_data($visitors_id);

  if ($hit_details) {
    $url     = urldecode($hit_details->visitors_url);
    $referer = $hit_details->visitors_referer;
    $date    = format_date(
      $hit_details->visitors_date_time,
      'large',
      visitors_get_timezone()
    );

    $whois_enable = module_exists('whois');
    $attr = array(
      'attributes' => array(
        'target' => '_blank',
        'title' => t('Whois lookup')
      )
    );
    $ip = long2ip($hit_details->visitors_ip);
    $array = array(
      'URL'            => l($url, $url),
      'Title'          => check_plain($hit_details->visitors_title),
      'Referer'        => $referer ? l($referer, $referer) : '',
      'Date'           => $date,
      'User'           => theme('username', array('account' => $hit_details)),
      'IP'             => $whois_enable ? l($ip, 'whois/' . $ip, $attr) : $ip,
      'User Agent'     => check_plain($hit_details->visitors_user_agent)
    );

    if (module_exists('visitors_geoip')) {
      $geoip_data = visitors_hit_details_get_geoip_data($hit_details);
      $array = array_merge($array, $geoip_data);
    }

    if (module_exists('visitors_browscap')) {
      $browscap_data = visitors_hit_details_get_browscap_data($hit_details);
      $array = array_merge($array, $browscap_data);
    }
    $rows = array();
    foreach ($array as $key => $value) {
      $rows[] = array(array('data' => t($key), 'header' => TRUE), $value);
    }

    drupal_set_title(t('Details'));
    return theme('table', array('rows' => $rows));
  }

  drupal_not_found();
}

/**
 * Get hit details data from visitors table.
 *
 * @param visitors_id
 *   int visitors id
 *
 * @return
 *   stdObject
 */
function visitors_hit_details_get_data($visitors_id) {
  $query = db_select('visitors', 'v');
  $query->leftJoin('users', 'u', 'u.uid=v.visitors_uid');
  $query->fields('v');
  $query->fields('u', array('name', 'uid'));
  $query->condition('v.visitors_id', (int) $visitors_id);
  return $query->execute()->fetch();
}

/**
 * Get geoip data from hit details.
 *
 * @param $hit_details
 *   stdObject
 *
 * @return
 *   array geoip data
 */
function visitors_hit_details_get_geoip_data($hit_details) {
  $geoip_data_array = array(
    'Country'        => check_plain($hit_details->visitors_country_name),
    'Region'         => check_plain($hit_details->visitors_region),
    'City'           => check_plain($hit_details->visitors_city),
    'Postal Code'    => check_plain($hit_details->visitors_postal_code),
    'Latitude'       => check_plain($hit_details->visitors_latitude),
    'Longitude'      => check_plain($hit_details->visitors_longitude),
    'DMA Code'       => check_plain($hit_details->visitors_dma_code),
    'PSTN Area Code' => check_plain($hit_details->visitors_area_code),
  );

  return $geoip_data_array;
}

/**
 * Get browscap data from hit details.
 *
 * @param $hit_details
 *   stdObject
 *
 * @return
 *   array browscap data
 */
function visitors_hit_details_get_browscap_data($hit_details) {
  $browscap_data_array = array(
    'Browser Name Regex'           => check_plain($hit_details->visitors_browser_name_regex),
    'Browser Name Pattern'         => check_plain($hit_details->visitors_browser_name_pattern),
    'Parent'                       => check_plain($hit_details->visitors_parent),
    'Platform'                     => check_plain($hit_details->visitors_platform),
    'Win32'                        => check_plain($hit_details->visitors_win32),
    'Comment'                      => check_plain($hit_details->visitors_comment),
    'Browser'                      => check_plain($hit_details->visitors_browser),
    'Version'                      => check_plain($hit_details->visitors_version),
    'Major Version'                => check_plain($hit_details->visitors_majorver),
    'Minor Version'                => check_plain($hit_details->visitors_minorver),
    'Frames'                       => check_plain($hit_details->visitors_frames),
    'IFrames'                      => check_plain($hit_details->visitors_iframes),
    'Tables'                       => check_plain($hit_details->visitors_tables),
    'Cookies'                      => check_plain($hit_details->visitors_cookies),
    'Java Script'                  => check_plain($hit_details->visitors_javascript),
    'Java Applets'                 => check_plain($hit_details->visitors_javaapplets),
    'CSS Version'                  => check_plain($hit_details->visitors_cssversion),
    'Rendering Engine Name'        => check_plain($hit_details->visitors_renderingengine_name),
    'Rendering Engine Description' => check_plain($hit_details->visitors_renderingengine_description),
    'Platform Version'             => check_plain($hit_details->visitors_platform_version),
    'Platform Description'         => check_plain($hit_details->visitors_platform_description),
    'Alpha'                        => check_plain($hit_details->visitors_alpha),
    'Beta'                         => check_plain($hit_details->visitors_beta),
    'Win16'                        => check_plain($hit_details->visitors_win16),
    'Win64'                        => check_plain($hit_details->visitors_win64),
    'Background Sounds'            => check_plain($hit_details->visitors_backgroundsounds),
    'VBScript'                     => check_plain($hit_details->visitors_vbscript),
    'Activex Controls'             => check_plain($hit_details->visitors_activexcontrols),
    'Is Mobile Device'             => check_plain($hit_details->visitors_ismobiledevice),
    'Is Syndication Reader'        => check_plain($hit_details->visitors_issyndicationreader),
    'Crawler'                      => check_plain($hit_details->visitors_crawler),
    'AOL Version'                  => check_plain($hit_details->visitors_aolversion),
    'Device Name'                  => check_plain($hit_details->visitors_device_name),
    'Device Maker'                 => check_plain($hit_details->visitors_device_maker),
    'Rendering Engine Version'     => check_plain($hit_details->visitors_renderingengine_version)
  );

  return $browscap_data_array;
}

